"""
This module contains the meta grants, which are grants that are not
meant to be used directly, but rather serve as a base for other grants.



"""

from .cache import CacheGrant

__all__ = ["CacheGrant"]
